@@ -26,7 +26,7 @@ from typing import TYPE_CHECKING, Iterable, Optional, Union
 from warnings import warn
 
 from . import _qpdf
-from ._qpdf import Object, ObjectType
+from ._qpdf import Object, ObjectType, Rectangle
 
 if TYPE_CHECKING:  # pragma: no cover
     from pikepdf import Pdf
@@ -39,12 +39,12 @@ ObjectType.__module__ = __name__
 
 
 # type(Object) is the metaclass that pybind11 defines; we wish to extend that
-# pylint cannot see the C++ metaclass definition is thoroughly confused.
+# pylint cannot see the C++ metaclass definition and is thoroughly confused.
 # pylint: disable=invalid-metaclass
 
 
 class _ObjectMeta(type(Object)):  # type: ignore
-    """Supports instance checking"""
+    """Supports instance checking."""
 
     def __instancecheck__(self, instance):
         # Note: since this class is a metaclass, self is a class object
@@ -54,7 +54,7 @@ class _ObjectMeta(type(Object)):  # type: ignore
 
 
 class _NameObjectMeta(_ObjectMeta):
-    """Supports usage pikepdf.Name.Whatever -> Name('/Whatever')"""
+    """Supports usage pikepdf.Name.Whatever -> Name('/Whatever')."""
 
     def __getattr__(self, attr):
         if attr.startswith('_'):
@@ -62,9 +62,13 @@ class _NameObjectMeta(_ObjectMeta):
         return Name('/' + attr)
 
     def __setattr__(self, attr, value):
-        if attr.startswith('_'):
-            return _ObjectMeta.__setattr__(attr, value)
-        raise TypeError("Attributes may not be set on pikepdf.Name")
+        # No need for a symmetric .startswith('_'). To prevent user error, we
+        # simply don't allow mucking with the pikepdf.Name class's attributes.
+        # There is no reason to ever assign to them.
+        raise AttributeError(
+            "Attributes may not be set on pikepdf.Name. Perhaps you meant to "
+            "modify a Dictionary rather than a Name?"
+        )
 
     def __getitem__(self, item):
         if item.startswith('/'):
@@ -78,7 +82,7 @@ class _NameObjectMeta(_ObjectMeta):
 
 
 class Name(Object, metaclass=_NameObjectMeta):
-    """Constructs a PDF Name object
+    """Constructs a PDF Name object.
 
     Names can be constructed with two notations:
 
@@ -101,15 +105,23 @@ class Name(Object, metaclass=_NameObjectMeta):
         return _qpdf._new_name(name)
 
     @classmethod
-    def random(cls, len_: int = 16, prefix: str = ''):
+    def random(cls, len_: int = 16, prefix: str = '') -> 'Name':
         """Generate a cryptographically strong random, valid PDF Name.
 
+        This function uses Python's secrets.token_urlsafe, which returns a
+        URL-safe encoded random number of the desired length. An optional
+        *prefix* may be prepended. (The encoding is ultimately done with
+        :func:`base64.urlsafe_b64encode`.) Serendipitiously, URL-safe is also
+        PDF-safe.
+
         When the length paramater is 16 (16 random bytes or 128 bits), the result
         is probably globally unique and can be treated as never colliding with
         other names.
+
+        The length of the string may vary because it is encoded.
         """
-        # It so happens that urlsafe names are also safe names for PDF Name objects
-        return _qpdf._new_name(f"/{prefix}{token_urlsafe(len_)}")
+        random_string = token_urlsafe(len_)
+        return _qpdf._new_name(f"/{prefix}{random_string}")
 
 
 class Operator(Object, metaclass=_ObjectMeta):
@@ -132,7 +144,7 @@ class Operator(Object, metaclass=_ObjectMeta):
 
 
 class String(Object, metaclass=_ObjectMeta):
-    """Constructs a PDF String object"""
+    """Constructs a PDF String object."""
 
     object_type = ObjectType.string
 
@@ -151,11 +163,11 @@ class String(Object, metaclass=_ObjectMeta):
 
 
 class Array(Object, metaclass=_ObjectMeta):
-    """Constructs a PDF Array object"""
+    """Constructs a PDF Array object."""
 
     object_type = ObjectType.array
 
-    def __new__(cls, a: Optional[Iterable] = None):
+    def __new__(cls, a: Optional[Union[Iterable, Rectangle]] = None):
         """
         Args:
             a: An iterable of objects. All objects must be either
@@ -167,15 +179,17 @@ class Array(Object, metaclass=_ObjectMeta):
 
         if isinstance(a, (str, bytes)):
             raise TypeError('Strings cannot be converted to arrays of chars')
-        if a is None:
+        elif a is None:
             a = []
-        if isinstance(a, Array):
+        elif isinstance(a, Rectangle):
+            return a.as_array()
+        elif isinstance(a, Array):
             return a.__copy__()
         return _qpdf._new_array(a)
 
 
 class Dictionary(Object, metaclass=_ObjectMeta):
-    """Constructs a PDF Dictionary object"""
+    """Constructs a PDF Dictionary object."""
 
     object_type = ObjectType.dictionary
 
@@ -210,13 +224,13 @@ class Dictionary(Object, metaclass=_ObjectMeta):
             return d.__copy__()
         if not d:
             d = {}
-        if d and any(not key.startswith('/') for key in d.keys()):
-            raise ValueError("Dictionary created from strings must begin with '/'")
+        if d and any(key == '/' or not key.startswith('/') for key in d.keys()):
+            raise KeyError("Dictionary created from strings must begin with '/'")
         return _qpdf._new_dictionary(d)
 
 
 class Stream(Object, metaclass=_ObjectMeta):
-    """Constructs a PDF Stream object"""
+    """Constructs a PDF Stream object."""
 
     object_type = ObjectType.stream
 
@@ -260,6 +274,9 @@ class Stream(Object, metaclass=_ObjectMeta):
                         b"data",
                         d
                     )
+
+        .. versionchanged:: 2.2
+            Support creation of ``pikepdf.Stream`` from existing dictionary.
         """
 
         # Support __new__(...obj=bytes) which should have been data=bytes,
